"""Extract interface."""

from Bio.PDB.Polypeptide import PPBuilder
from Bio.PDB.NeighborSearch import NeighborSearch
from Bio.PDB import Structure, Model, Chain, Residue
from Bio.PDB import MMCIFIO

import pandas as pd

from PDBNucleicAcids.NucleicAcid import NABuilder


def get_interface(structure, protein_chain_id, search_radius, n_atoms):
    """Extract interface."""
    # get all the atoms from the nucleic acids, in most cases DNA
    na_builder = NABuilder()
    na_list = na_builder.build_nucleic_acids(structure)
    na_atoms = []
    for na in na_list:
        na_atoms.extend(na.get_atoms())
    na_atoms = list(set(na_atoms))

    # get all the atoms from the protein chain
    protein_chain = structure[0][protein_chain_id]
    pp_builder = PPBuilder()
    pp = pp_builder.build_peptides(protein_chain)[0]
    pp_atoms = []
    for pp in pp_builder.build_peptides(protein_chain):
        for res in pp:
            pp_atoms.extend(res.get_atoms())
    pp_atoms = list(set(pp_atoms))

    # Crea una lista con tutti gli atomi di DNA e proteina
    all_atoms = na_atoms + pp_atoms

    # Usa NeighborSearch per trovare gli atomi vicini entro una certa
    # distanza
    ns = NeighborSearch(all_atoms)

    # Cerca gli atomi vicini entro un raggio di 4 Å tra DNA e proteina
    all_contacts = ns.search_all(search_radius)

    # Filtra solo i contatti, ovvero le coppied di atomi,
    # che hanno un atomo di DNA
    temp = [
        (atom1, atom2)
        for atom1, atom2 in all_contacts
        if (atom1 in na_atoms or atom2 in na_atoms)
    ]

    # Filtra solo i contatti tra DNA e proteina
    contacts = [
        (atom1, atom2)
        for atom1, atom2 in temp
        if (atom1 in na_atoms and atom2 in pp_atoms)
    ] + [
        (atom2, atom1)
        for atom1, atom2 in temp
        if (atom1 in pp_atoms and atom2 in na_atoms)
    ]
    # temp è utile per greedyness, prima prendi il DNA, che ha meno
    # atomi, escludendo quelli intra-proteina
    # poi escludi anche quelli intra-DNA
    # inoltre ci assicuriamo che la col 0 contenga gli atomi di DNA e
    # che col 1 contenga gli atomi di proteine
    # Ce ne assicuriamo invertendo atom1 e atom2 nella seconda lista

    # dataframe
    contacts_df = pd.DataFrame(contacts, columns=["na_atoms", "pp_atoms"])

    # distance column
    # Atom - Atom = distance in armstrong
    contacts_df["dist"] = contacts_df.apply(
        lambda row: row["na_atoms"] - row["pp_atoms"], axis=1
    )

    # grouping: minimum distance for every protein atom, then sorted
    dist = contacts_df.groupby(by="pp_atoms")["dist"].min().sort_values()

    # take the n top smallest distances
    # atoms as indexes
    dist = dist.head(n_atoms)  # Series of floats
    interface_atoms = dist.index.tolist()  # index of Atom objects, to list

    return interface_atoms


def export_atom_list(structure_id, atom_list, out_filepath):
    # not in Path but in string
    out_filepath = str(out_filepath)

    new_structure = Structure.Structure(structure_id)
    for atom in atom_list:
        _add_atom_to_new_structure(atom, new_structure)

    # Prepare IO object
    io = MMCIFIO()
    io.set_structure(new_structure)

    # Esporta la nuova struttura in un file PDB
    # necessita di string type filepath
    io.save(out_filepath)


def _add_atom_to_new_structure(atom, new_structure):
    model_id = (
        atom.get_parent().get_parent().get_parent().id
    )  # Ottieni l'ID del modello
    chain_id = atom.get_parent().get_parent().id  # Ottieni l'ID della catena
    residue_id = atom.get_parent().id  # Ottieni l'ID del residuo
    resname = atom.get_parent().resname  # Nome del residuo

    # Controlla se il modello esiste già nel nuovo modello
    if model_id in [model.id for model in new_structure]:
        new_model = new_structure[model_id]
    else:
        new_model = Model.Model(model_id)
        new_structure.add(new_model)

    # Controlla se la catena esiste già nel nuovo modello
    if chain_id in [chain.id for chain in new_model]:
        new_chain = new_model[chain_id]
    else:
        new_chain = Chain.Chain(chain_id)
        new_model.add(new_chain)

    # Controlla se il residuo esiste già nella nuova catena
    if residue_id in [res.id for res in new_chain]:
        new_residue = new_chain[residue_id]
    else:
        new_residue = Residue.Residue(
            residue_id, resname, atom.get_parent().segid
        )
        new_chain.add(new_residue)

    # Copia l'atomo e aggiungilo al residuo
    new_atom = atom.copy()
    new_residue.add(new_atom)
